@@ -129,9 +129,10 @@ class Converter(object):
     def all_formats(self):
         return self._FORMAT_INFO
 
-    def __init__(self, config, project_dir, output_tags=None, upload_dir=None):
+    def __init__(self, config, project_dir, output_tags=None, upload_dir=None, download_resources=True):
         self.project_dir = project_dir
         self.upload_dir = upload_dir
+        self.download_resources = download_resources
         if isinstance(config, dict):
             self._schema = config
         elif isinstance(config, str):
@@ -184,7 +185,7 @@ class Converter(object):
             items = self.iter_from_dir(input_data) if is_dir else self.iter_from_json_file(input_data)
             convert_to_asr_json_manifest(
                 items, output_data, data_key=self._data_keys[0], project_dir=self.project_dir,
-                upload_dir=self.upload_dir)
+                upload_dir=self.upload_dir, download_resources=self.download_resources)
 
     def _get_data_keys_and_output_tags(self, output_tags=None):
         data_keys = set()
@@ -306,7 +307,10 @@ class Converter(object):
                 'input': inputs,
                 'output': outputs,
                 'completed_by': annotation.get('completed_by', {}),
-                'annotation_id': annotation.get('id')
+                'annotation_id': annotation.get('id'),
+                'created_at': annotation.get('created_at'),
+                'updated_at': annotation.get('updated_at'),
+                'lead_time': annotation.get('lead_time')
             }
 
     def _check_format(self, fmt):
@@ -320,9 +324,11 @@ class Converter(object):
             tag_type = j.pop('type')
             if tag_type == 'Choices' and len(j['choices']) == 1:
                 out.append(j['choices'][0])
+            elif tag_type == 'TextArea' and len(j['text']) == 1:
+                out.append(j['text'][0])
             else:
                 out.append(j)
-        return out[0] if tag_type == 'Choices' and len(out) == 1 else out
+        return out[0] if tag_type in ('Choices', 'TextArea') and len(out) == 1 else out
 
     def convert_to_json(self, input_data, output_dir, is_dir=True):
         self._check_format(Format.JSON)
@@ -353,6 +359,9 @@ class Converter(object):
                 record[name] = self._prettify(value)
             record['annotator'] = item['completed_by'].get('email')
             record['annotation_id'] = item['annotation_id']
+            record['created_at'] = item['created_at']
+            record['updated_at'] = item['updated_at']
+            record['lead_time'] = item['lead_time']
             records.append(record)
 
         with io.open(output_file, mode='w', encoding='utf8') as fout:
@@ -374,6 +383,9 @@ class Converter(object):
                 record[name] = pretty_value if isinstance(pretty_value, str) else json.dumps(pretty_value)
             record['annotator'] = item['completed_by'].get('email')
             record['annotation_id'] = item['annotation_id']
+            record['created_at'] = item['created_at']
+            record['updated_at'] = item['updated_at']
+            record['lead_time'] = item['lead_time']
             records.append(record)
 
         pd.DataFrame.from_records(records).to_csv(output_file, index=False, **kwargs)
@@ -406,7 +418,7 @@ class Converter(object):
             output_image_dir = os.path.join(output_dir, 'images')
             os.makedirs(output_image_dir, exist_ok=True)
         images, categories, annotations = [], [], []
-        category_name_to_id = {}
+        categories, category_name_to_id = self._get_labels()
         data_key = self._data_keys[0]
         item_iterator = self.iter_from_dir(input_data) if is_dir else self.iter_from_json_file(input_data)
         for item_idx, item in enumerate(item_iterator):
@@ -417,7 +429,8 @@ class Converter(object):
             if not os.path.exists(image_path):
                 try:
                     image_path = download(image_path, output_image_dir, project_dir=self.project_dir,
-                                          return_relative_path=True, upload_dir=self.upload_dir)
+                                          return_relative_path=True, upload_dir=self.upload_dir,
+                                          download_resources=self.download_resources)
                 except:
                     logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item
@@ -457,13 +470,14 @@ class Converter(object):
                     })
                     first = False
 
-                if category_name not in category_name_to_id:
+                '''if category_name not in category_name_to_id:
                     category_id = len(categories)
                     category_name_to_id[category_name] = category_id
                     categories.append({
                         'id': category_id,
-                        'name': category_name
-                    })
+                        'name': category_name,
+                        'supercategory': category_name
+                    })'''
                 category_id = category_name_to_id[category_name]
 
                 annotation_id = len(annotations)
@@ -543,7 +557,8 @@ class Converter(object):
             if not os.path.exists(image_path):
                 try:
                     image_path = download(image_path, output_image_dir, project_dir=self.project_dir,
-                                          return_relative_path=True, upload_dir=self.upload_dir)
+                                          return_relative_path=True, upload_dir=self.upload_dir,
+                                          download_resources=self.download_resources)
                 except:
                     logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item
@@ -636,7 +651,7 @@ class Converter(object):
                 try:
                     image_path = download(
                         image_path, output_image_dir, project_dir=self.project_dir,
-                        upload_dir=self.upload_dir, return_relative_path=True)
+                        upload_dir=self.upload_dir, return_relative_path=True, download_resources=self.download_resources)
                 except:
                     logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item), exc_info=True)
@@ -715,3 +730,34 @@ class Converter(object):
 
             with io.open(xml_filepath, mode='w', encoding='utf8') as fout:
                 doc.writexml(fout, addindent='' * 4, newl='\n', encoding='utf-8')
+
+    def _get_labels(self):
+        labels = set()
+        categories = list()
+        category_name_to_id = dict()
+
+        for name, info in self._schema.items():
+            labels |= set(info['labels'])
+            attrs = info['labels_attrs']
+            for label in attrs:
+                if attrs[label].get('category'):
+                    categories.append({
+                        'id': attrs[label].get('category'),
+                        'name': label
+                    })
+                    category_name_to_id[label] = attrs[label].get('category')
+        labels_to_add = set(labels) - set(list(category_name_to_id.keys()))
+        labels_to_add = sorted(list(labels_to_add))
+        idx = 0
+        while idx in list(category_name_to_id.values()):
+            idx += 1
+        for label in labels_to_add:
+            categories.append({
+                'id': idx,
+                'name': label
+            })
+            category_name_to_id[label] = idx
+            idx += 1
+            while idx in list(category_name_to_id.values()):
+                idx += 1
+        return categories, category_name_to_id
\ No newline at end of file
